package cd.casic.ci.common.pipeline;

import cd.casic.ci.common.pipeline.functions.ExpressionOutput;
import cd.casic.ci.common.pipeline.functions.IFunctionInfo;
import cd.casic.ci.common.pipeline.pojo.NamedValueInfo;
import cd.casic.ci.common.pipeline.dialect.IPipelineDialect;
import cd.casic.ci.common.pipeline.utils.ObjectReplaceEnvVarUtil;
import cn.hutool.core.lang.Pair;
import cn.hutool.json.JSONUtil;
import com.networknt.schema.ExecutionContext;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline
 * @Project：ops-pro
 * @name：EnvReplacementParser
 * @Date：2025/03/25 17:38
 * @Filename：EnvReplacementParser
 * @description：Todo
 */
@Slf4j
public class EnvReplacementParser {
    private static final Pattern expressionPattern = Pattern.compile("\\$[{]{2}([^$^{}]+)[}]{2}");

    public static String parse(Object value, Map<String, String> contextMap, Boolean onlyExpression,
                               Pair<ExecutionContext, List<NamedValueInfo>> contextPair, Iterable<IFunctionInfo> functions,
                               ExpressionOutput output) {
        ExprReplacementOptions options = new ExprReplacementOptions(contextMap, contextPair, functions, output);
        return parse(value, onlyExpression, options);
    }

    public static String parse(Object value, Map<String, String> contextMap, IPipelineDialect dialect,
                               Pair<ExecutionContext, List<NamedValueInfo>> contextPair, Iterable<IFunctionInfo> functions,
                               ExpressionOutput output) {
        ExprReplacementOptions options = new ExprReplacementOptions(contextMap, contextPair, functions, output);
        return parse(value, dialect.supportUseExpression(), options);
    }

    public static String parse(Object value, Boolean onlyExpression, ExprReplacementOptions options) {
        if (value == null) return "";
        if (onlyExpression == null || !onlyExpression) {
            return JSONUtil.toJsonStr(ObjectReplaceEnvVarUtil.replaceEnvVar(value, options.getContextMap()));
        } else {
            //return JsonUtil.toJson(ExprReplacementUtil.replaceEnvVar(value, options), false);
            return null;
        }
    }

//    public static Pair<ExecutionContext, List<NamedValueInfo>> getCustomExecutionContextByMap(Map<String, String> variables,
//                                                                                              List<RuntimeNamedValue> extendNamedValueMap) {
//        return ExprReplacementUtil.getCustomExecutionContextByMap(variables, extendNamedValueMap);
//    }

    public static boolean containsExpressions(String value) {
        if (value == null) {
            return false;
        }
        return expressionPattern.matcher(value).find();
    }
}
