package cd.casic.ci.common.pipeline.option;

import cd.casic.ci.common.pipeline.NameAndValue;
import cd.casic.ci.common.pipeline.enums.DependOnType;
import cd.casic.ci.common.pipeline.enums.JobRunCondition;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.util.List;
import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.option
 * @Project：ops-pro
 * @name：JobControlOption
 * @Date：2025/03/25 17:59
 * @Filename：JobControlOption
 * @description：Todo
 */
@Data
@Accessors(chain = true)
@Schema(title = "job流程控制模型")
@NoArgsConstructor
@AllArgsConstructor
public class JobControlOption {
    @Schema(title = "是否启用Job", required = false)
    private Boolean enable = true;
    @Schema(title = "Job准备环境的超时时间 分钟Minutes", required = false)
    private Integer prepareTimeout;
    @Schema(title = "Job执行的超时时间 分钟Minutes", required = false)
    private Integer timeout = 900;
    @Schema(title = "新的Job执行的超时时间，支持变量 分钟Minutes，出错则取timeout的值", required = false)
    private String timeoutVar;
    @Schema(title = "运行条件", required = false)
    private JobRunCondition runCondition = JobRunCondition.STAGE_RUNNING;
    @Schema(title = "自定义变量", required = false)
    private List<NameAndValue> customVariables = List.of();
    @Schema(title = "自定义条件", required = false)
    private String customCondition;
    @Schema(title = "job依赖", required = false)
    private DependOnType dependOnType;
    @Schema(title = "需要过滤不存在的job，定义为var类型", required = false)
    private List<String> dependOnId;
    @Schema(title = "job依赖名称", required = false)
    private String dependOnName;
    @Schema(title = "containerId与jobId映射，depend on运行时使用的是containerId", required = false)
    private Map<String, String> dependOnContainerId2JobIds;
    @Schema(title = "是否失败继续", required = false)
    private Boolean continueWhenFailed = false;
    @Schema(title = "第三方构建机集群-单节点并发限制")
    private Integer singleNodeConcurrency;
    @Schema(title = "第三方构建机集群-所有节点并发限制")
    private Integer allNodeConcurrency;

    public JobControlOption(boolean enable, Integer maxRunningMinutes, String s, JobRunCondition jobRunCondition, List<NameAndValue> conditions) {
        this.enable = enable;
        this.prepareTimeout = maxRunningMinutes;
        this.runCondition = jobRunCondition;
        this.customVariables = conditions;

    }
}
