package cd.casic.ci.common.pipeline.pojo.element.trigger;

import cd.casic.ci.common.pipeline.enums.StartType;
import cd.casic.ci.common.pipeline.enums.TriggerRepositoryType;
import cd.casic.ci.common.pipeline.pojo.element.ElementProp;
import cd.casic.ci.common.pipeline.pojo.element.trigger.enums.CodeEventType;
import cd.casic.ci.common.pipeline.pojo.element.trigger.enums.PathFilterType;
import cd.casic.ci.common.pipeline.utils.TriggerElementPropUtils;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.experimental.Accessors;

import java.util.List;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.element.trigger
 * @Project：ops-pro
 * @name：CodeGitlabWebHookTriggerElement
 * @Date：2025/03/27 10:22
 * @Filename：CodeGitlabWebHookTriggerElement
 * @description：Todo
 */
@Accessors(chain = true)
@Schema(title = "GitLab仓库代码提交触发", description = CodeGitlabWebHookTriggerElement.classType)
public class CodeGitlabWebHookTriggerElement extends WebHookTriggerElement {
    public static final String classType = "codeGitlabWebHookTrigger";

    @Schema(title = "任务名称", required = true)
    private String name = "Gitlab变更触发";

    @Schema(title = "id", required = false)
    private String id;

    @Schema(title = "状态", required = false)
    private String status;

    @Schema(title = "仓库ID", required = true)
    private String repositoryHashId;

    @Schema(title = "分支名称", required = false)
    private String branchName;

    @Schema(title = "新版的gitlab原子的类型")
    private TriggerRepositoryType repositoryType;

    @Schema(title = "新版的gitlab代码库名")
    private String repositoryName;

    @Schema(title = "事件类型", required = false)
    private CodeEventType eventType = CodeEventType.PUSH;

    @Schema(title = "用于排除的分支名", required = false)
    private String excludeBranchName;

    @Schema(title = "路径过滤类型", required = true)
    private PathFilterType pathFilterType = PathFilterType.NamePrefixFilter;

    @Schema(title = "用于包含的路径", required = false)
    private String includePaths;

    @Schema(title = "用于排除的路径", required = false)
    private String excludePaths;

    @Schema(title = "用于包含的user id", required = false)
    private List<String> includeUsers;

    @Schema(title = "用于排除的user id", required = false)
    private List<String> excludeUsers;

    @Schema(title = "是否为block", required = false)
    private Boolean block;

    @Schema(title = "tag名称", required = false)
    private String tagName;

    @Schema(title = "用于排除的tag名称", required = false)
    private String excludeTagName;

    @Schema(title = "用于排除的源分支名称", required = false)
    private String excludeSourceBranchName;

    @Schema(title = "用于包含的源分支名称", required = false)
    private String includeSourceBranchName;

    @Schema(title = "用于包含的提交信息", required = false)
    private String includeCommitMsg;

    @Schema(title = "用于排除的提交信息", required = false)
    private String excludeCommitMsg;

    @Schema(title = "push事件action")
    private List<String> includePushAction;

    @Schema(title = "mr事件action")
    private List<String> includeMrAction;

    public CodeGitlabWebHookTriggerElement(String name, String id, String status) {
        super(name, id, status);
    }

    @Override
    public String getClassType() {
        return classType;
    }

    @Override
    public String findFirstTaskIdByStartType(StartType startType) {
        return StartType.WEB_HOOK.name().equals(startType.name()) ? this.id : super.findFirstTaskIdByStartType(startType);
    }

    @Override
    public List<ElementProp> triggerCondition() {
        List<ElementProp> props;
        switch (eventType) {
            case PUSH:
                props = List.of(
                        TriggerElementPropUtils.vuexInput("action", joinToString(includePushAction)),
                        TriggerElementPropUtils.vuexInput("branchName", branchName),
                        TriggerElementPropUtils.vuexInput("excludeBranchName", excludeBranchName),
                        TriggerElementPropUtils.vuexInput("includePaths", includePaths),
                        TriggerElementPropUtils.vuexInput("excludePaths", excludePaths),
                        TriggerElementPropUtils.staffInput("includeUsers", includeUsers),
                        TriggerElementPropUtils.staffInput("excludeUsers", excludeUsers)
                );
                break;
            case MERGE_REQUEST:
                props = List.of(
                        TriggerElementPropUtils.vuexInput("action", joinToString(includeMrAction)),
                        TriggerElementPropUtils.vuexInput("branchName", branchName),
                        TriggerElementPropUtils.vuexInput("excludeBranchName", excludeBranchName),
                        TriggerElementPropUtils.vuexInput("includeSourceBranchName", includeSourceBranchName),
                        TriggerElementPropUtils.vuexInput("includeSourceBranchName", includeSourceBranchName),
                        TriggerElementPropUtils.vuexInput("includePaths", includePaths),
                        TriggerElementPropUtils.vuexInput("excludePaths", excludePaths),
                        TriggerElementPropUtils.staffInput("includeUsers", includeUsers),
                        TriggerElementPropUtils.staffInput("excludeUsers", excludeUsers)
                );
                break;
            case MERGE_REQUEST_ACCEPT:
                props = List.of(
                        TriggerElementPropUtils.vuexInput("action", "merge"),
                        TriggerElementPropUtils.vuexInput("branchName", branchName),
                        TriggerElementPropUtils.vuexInput("excludeBranchName", excludeBranchName),
                        TriggerElementPropUtils.vuexInput("includeSourceBranchName", includeSourceBranchName),
                        TriggerElementPropUtils.vuexInput("includeSourceBranchName", includeSourceBranchName),
                        TriggerElementPropUtils.vuexInput("includePaths", includePaths),
                        TriggerElementPropUtils.vuexInput("excludePaths", excludePaths),
                        TriggerElementPropUtils.staffInput("includeUsers", includeUsers),
                        TriggerElementPropUtils.staffInput("excludeUsers", excludeUsers)
                );
                break;
            case TAG_PUSH:
                props = List.of(
                        TriggerElementPropUtils.vuexInput("tagName", tagName),
                        TriggerElementPropUtils.vuexInput("excludeTagName", excludeTagName)
                );
                break;
            default:
                props = List.of();
        }
        return props.stream().filter(java.util.Objects::nonNull).toList();
    }
}
