package cd.casic.ci.common.pipeline.utils;

import cd.casic.framework.commons.util.reflect.ReflectUtil;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.yaml.YAMLFactory;
import com.fasterxml.jackson.dataformat.yaml.YAMLGenerator;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import lombok.Getter;
import org.yaml.snakeyaml.Yaml;

import java.lang.reflect.Method;

/**
 * @author by mianbin
 * @Classname YamlUtil
 * @Description TODO
 * @Date 2025/3/29 16:13
 */
@Getter
public class YamlUtil {
    public static final ObjectMapper objectMapper = new ObjectMapper(new YAMLFactory().disable(YAMLGenerator.Feature.SPLIT_LINES)).registerModule(new JavaTimeModule());

    private static final int RETRY_TIME = 3;

    public static String toYaml(Object bean) {
        if (isNativeType(bean) || bean instanceof String) {
            return bean.toString();
        }
        try {
            return getObjectMapper().writeValueAsString(bean);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private static ObjectMapper getObjectMapper() {
        return objectMapper;
    }

    public static <T> T to(String yamlStr, TypeReference<T> valueTypeRef) {
        if (valueTypeRef == null) {
            valueTypeRef = new TypeReference<T>() {
            };
        }
        String obj = loadYamlRetryOnAccident(yamlStr);
        try {
            return getObjectMapper().readValue(obj, valueTypeRef);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public static String loadYamlRetryOnAccident(String yamlStr, int retryTime, long retryPeriodMills) {
        try {
            String yml = (RETRY_TIME != retryTime) ? "# auto fix " + "+".repeat(retryTime) + "\n" + yamlStr : yamlStr;
            Yaml yaml = new Yaml();
            return toYaml(yaml.load(yml));
        } catch (Throwable re) {
            if (retryTime - 1 < 0) {
                throw new RuntimeException(re);
            }
            if (retryPeriodMills > 0) {
                try {
                    Thread.sleep(retryPeriodMills);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }
            }
            return loadYamlRetryOnAccident(yamlStr, retryTime - 1, retryPeriodMills);
        }
    }

    public static String loadYamlRetryOnAccident(String yamlStr) {
        return loadYamlRetryOnAccident(yamlStr, RETRY_TIME, 50);
    }

    private static boolean isNativeType(Object obj) {
        try {
            Method method = ReflectUtil.class.getMethod("isNativeType", Object.class);
            return (boolean) method.invoke(null, obj);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
