package cd.casic.ci.log.jmx;

import lombok.Synchronized;
import org.springframework.jmx.export.annotation.ManagedAttribute;
import org.springframework.jmx.export.annotation.ManagedResource;
import org.springframework.stereotype.Component;

import java.util.concurrent.atomic.AtomicLong;

/**
 * @author by mianbin
 * @Classname CreateIndexBean
 * @Description TODO
 * @Date 2025/3/20 10:25
 */
@Component
@ManagedResource(objectName = "ops.ci.log.v2:type=index,operation=create", description = "log create index performance")
public class CreateIndexBean {
    private AtomicLong executeCount = new AtomicLong(0);
    private AtomicLong executeElapse = new AtomicLong(0);
    private AtomicLong calculateCount = new AtomicLong(0);
    private AtomicLong failureCount = new AtomicLong(0);

    @Synchronized
    public void execute(long elapse, boolean success) {
        executeCount.incrementAndGet();
        calculateCount.incrementAndGet();
        executeElapse.addAndGet(elapse);
        if (!success) {
            failureCount.incrementAndGet();
        }
    }

    @Synchronized
    @ManagedAttribute
    public double getCreateIndexPerformance() {
        long elapse = executeElapse.getAndSet(0);
        long count = calculateCount.getAndSet(0);
        if (count == 0L) {
            return 0L;
        } else {
            return (double) elapse / count;
        }
    }

    @ManagedAttribute
    public long getExecuteCount() {
        return executeCount.get();
    }

    @ManagedAttribute
    public long getFailureCount() {
        return failureCount.get();
    }
}
