package cd.casic.ci.process.engine.worker;

import cd.casic.ci.common.pipeline.annotation.Plugin;
import cd.casic.ci.process.engine.constant.DIYImageExecuteCommandConstant;
import cd.casic.ci.process.engine.runContext.TaskRunContext;
import cd.casic.ci.process.process.dataObject.base.PipBaseElement;
import cd.casic.ci.process.process.dataObject.machine.MachineInfo;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 自定义镜像执行命令
 *
 * @author herenbin
 * @date 2022-11-08 9:59
 */
@Slf4j
@Plugin(taskType = "CUSTOM_IMAGE_EXECUTION_COMMAND")
public class DIYImageExecuteCommandWorker extends SshWorker {



    @Override
    public void execute(TaskRunContext context) {
        int statusCode = -1;
        AtomicInteger state = context.getState();
        if (context.getContextDef() instanceof PipTask taskDef) {
            log.info(taskDef.getTaskName());
            Map<String, Object> taskProperties = taskDef.getTaskProperties();
            Object commandScriptObj = taskProperties.get(DIYImageExecuteCommandConstant.COMMAND_SCRIPT);
            Object machineIdObj = taskProperties.get(DIYImageExecuteCommandConstant.MACHINE_ID);
            String commandScript = commandScriptObj instanceof String ? ((String) commandScriptObj) : null;
            Long machineId = null;
            try {
                machineId=Long.valueOf(String.valueOf(machineIdObj));
            } catch (NumberFormatException e) {
                log.error("缺少参数：{}",DIYImageExecuteCommandConstant.MACHINE_ID);
            }
            if (StringUtils.isEmpty(commandScript) ||machineIdObj == null) {
                throw new ServiceException(GlobalErrorCodeConstants.PIPELINE_ERROR.getCode(),"缺少参数");
            }

            try {
                //将节点的配置信息反编译成对象
                log.info("构建脚本" + commandScript);

                //如果machineId为0，则说明该节点没有配置机器，则使用开始节点的机器

                //获取机器
                MachineInfo machineInfoDO = this.getMachineInfoService().getById(machineId);
                statusCode = shell(machineInfoDO,
                        "echo \"自定义镜像执行命令\"",
                        commandScript
                );
            } catch (Exception e) {
                String errorMessage = "该节点配置信息为空，请先配置该节点信息" + "\r\n";
                log.error("执行ssh失败:",e);
//                errorHandle(e, errorMessage);
            }
            if (statusCode == 0) {
//                log.info("{}节点执行完成", getName());
            } else {
//                log.error("{}节点执行失败", getName());
            }
            context.getLocalVariables().put(DIYImageExecuteCommandConstant.STATUS_CODE,statusCode);
        }
    }
}
