package cd.casic.pipeline.store.api.atom;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.pipeline.store.pojo.atom.*;
import cd.casic.pipeline.store.pojo.atom.enums.AtomTypeEnum;
import cd.casic.pipeline.store.pojo.atom.enums.MarketAtomSortTypeEnum;
import cd.casic.pipeline.store.pojo.common.InstalledProjRespItem;
import cd.casic.pipeline.store.pojo.common.MarketMainItem;
import cd.casic.pipeline.store.pojo.common.StoreErrorCodeInfo;
import cd.casic.pipeline.store.pojo.common.version.StoreShowVersionInfo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;


@Tag(name = "USER_MARKET_ATOM", description = "插件市场-插件")
@RestController
@RequestMapping("/user/market/")
public interface UserMarketAtomResource {

    @Operation(summary = "获取插件市场首页的数据")
    @GetMapping("/atom/list/main")
    CommonResult<List<MarketMainItem>> mainPageList(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "插件市场搜索插件")
    @GetMapping("/atom/list/")
    CommonResult<MarketAtomResp> list(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "搜索关键字", required = false)
            @RequestParam("keyword")
            String keyword,
            @Parameter(description = "插件分类", required = false)
            @RequestParam("classifyCode")
            String classifyCode,
            @Parameter(description = "功能标签", required = false)
            @RequestParam("labelCode")
            String labelCode,
            @Parameter(description = "评分", required = false)
            @RequestParam("score")
            Integer score,
            @Parameter(description = "研发来源", required = false)
            @RequestParam("rdType")
            AtomTypeEnum rdType,
            @Parameter(description = "yaml是否可用", required = false)
            @RequestParam("yamlFlag")
            Boolean yamlFlag,
            @Parameter(description = "是否推荐标识 true：推荐，false：不推荐", required = false)
            @RequestParam("recommendFlag")
            Boolean recommendFlag,
            @Parameter(description = "是否有红线指标", required = false)
            @RequestParam("qualityFlag")
            Boolean qualityFlag,
            @Parameter(description = "排序", required = false)
            @RequestParam("sortType")
            MarketAtomSortTypeEnum sortType,
            @Parameter(description = "页码", required = false)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = false)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "根据用户获取插件工作台插件列表")
    @GetMapping("/desk/atom/list/")
    CommonResult<MyAtomResp> listMyAtoms(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "插件名称", required = false)
            @RequestParam("atomName")
            String atomName,
            @Parameter(description = "页码", required = true)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = true)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "根据插件版本ID获取插件详情")
    @GetMapping("/desk/atom/{atomId}")
    CommonResult<AtomVersion> getAtomById(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "atomId", required = true)
            @PathVariable("atomId")
            String atomId
    );

    @Operation(summary = "根据插件标识获取插件正式版本详情")
    @GetMapping("/atom/{atomCode}")
    CommonResult<AtomVersion> getAtomByCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "atomCode", required = true)
            @PathVariable("atomCode")
            String atomCode
    );

    @Operation(summary = "根据插件标识获取插件版本列表")
    @GetMapping("/atom/version/list/")
    PageResult<AtomVersionListItem> getAtomVersionsByCode(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "atomCode", required = true)
            @RequestParam("atomCode")
            String atomCode,
            @Parameter(description = "页码", required = true)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = true)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "安装插件到项目")
    @PostMapping("/atom/install")
    CommonResult<Boolean> installAtom(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "安装插件到项目请求报文体", required = true)
            InstallAtomReq installAtomReq
    );

    @Operation(summary = "根据插件标识获取已安装的项目列表")
    @GetMapping("/atom/installedProjects/{atomCode}")
    CommonResult<List<InstalledProjRespItem>> getInstalledProjects(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "atomCode", required = true)
            @PathVariable("atomCode")
            String atomCode
    );

    @Operation(summary = "获取插件支持的语言列表")
    @GetMapping("/desk/atom/language")
    CommonResult<List<AtomDevLanguage>> listLanguage();

    @Operation(summary = "删除工作台插件")
    @DeleteMapping("/desk/atoms/{atomCode}")
    CommonResult<Boolean> deleteAtom(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "插件代码", required = true)
            @PathVariable("atomCode")
            String atomCode
    );

    @Operation(summary = "根据插件标识获取插件回显版本信息")
    @GetMapping("/atoms/{atomCode}/showVersionInfo")
    CommonResult<StoreShowVersionInfo> getAtomShowVersionInfo(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "atomCode", required = true)
            @PathVariable("atomCode")
            String atomCode
    );

    @Operation(summary = "查看插件的yml信息")
    @GetMapping("/atoms/{atomCode}/yml/detail")
    CommonResult<String> getAtomYmlInfo(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "atomCode", required = true)
            @PathVariable("atomCode")
            String atomCode,
            @Parameter(description = "是否展示系统自带的yml信息", required = false)
            @RequestParam("defaultShowFlag")
            Boolean defaultShowFlag
    );

    @Operation(summary = "查看插件的yml 2.0信息")
    @GetMapping("/atoms/{atomCode}/yml/2.0/detail")
    CommonResult<String> getAtomYmlV2Info(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "atomCode", required = true)
            @PathVariable("atomCode")
            String atomCode,
            @Parameter(description = "是否展示系统自带的yml信息", required = false)
            @RequestParam("defaultShowFlag")
            Boolean defaultShowFlag
    );

    @Operation(summary = "展示插件的outPut参数")
    @GetMapping("/atoms/{atomCode}/output")
    CommonResult<List<AtomOutput>> getAtomOutput(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "atomCode", required = true)
            @PathVariable("atomCode")
            String atomCode
    );

    @Operation(summary = "更新插件自定义错误码信息")
    @PutMapping("/{projectCode}/atom/errorCodeInfo")
    CommonResult<Boolean> updateAtomErrorCodeInfo(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "projectCode", required = true)
            @PathVariable("projectCode")
            String projectCode,
            @Parameter(description = "插件自定义错误码信息", required = true)
            StoreErrorCodeInfo storeErrorCodeInfo
    );
}