package cd.casic.pipeline.store.api.container;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.container.Container;
import cd.casic.pipeline.store.pojo.container.ContainerRequest;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Tag(name = "OP_PIPELINE_CONTAINER", description = "OP-流水线-构建容器")
@RestController
@RequestMapping("/op/pipeline/container")
public interface OpContainerResource {

    @Operation(summary = "添加流水线构建容器")
    @PostMapping("/")
    CommonResult<Boolean> add(
            @Parameter(description = "流水线构建容器请求报文体", required = true)
            ContainerRequest pipelineContainerRequest
    );

    @Operation(summary = "更新流水线构建容器信息")
    @PutMapping("/{id}")
    CommonResult<Boolean> update(
            @Parameter(description = "容器ID", required = true)
            @PathVariable("id")
            String id,
            @Parameter(description = "流水线构建容器请求报文体", required = true)
            ContainerRequest pipelineContainerRequest
    );

    @Operation(summary = "获取所有的流水线构建容器信息")
    @GetMapping("/")
    CommonResult<List<Container>> listAllContainers();

    @Operation(summary = "根据ID获取流水线构建容器信息")
    @GetMapping("/{id}")
    CommonResult<Container> getContainerById(
            @Parameter(description = "容器ID", required = true)
            @PathVariable("id")
            String id
    );

    @Operation(summary = "根据ID删除流水线构建容器信息")
    @DeleteMapping("/{id}")
    CommonResult<Boolean> deleteContainerById(
            @Parameter(description = "容器ID", required = true)
            @PathVariable("id")
            String id
    );
}